<?php

namespace Drupal\drulma\Render;

use Drupal\Core\Render\Element;

/**
 * Provides a trusted callbacks to alter some elements markup.
 *
 * NOTE:
 * When dropping compatibility for Drupal < 8.8.0 this object
 * needs to implement
 * Drupal\Core\Render\Element\RenderCallbackInterface;
 *
 * @see https://www.drupal.org/node/2966725
 *
 * @see https://git.drupalcode.org/project/drupal/commit/5a42b47b6ed88a3bf65ff8d23ee24f49ccd20b61#7597429dcbbf29cc3aaf9f732750eeb93a0e61a5_0_1
 *
 * @see drulma_element_info_alter()
 */
class Callback {

  /**
   * Wrap all children with a panel block div.
   *
   * #pre_render callback.
   */
  public static function preRenderDetails($element) {
    foreach (Element::getVisibleChildren($element) as $child) {
      // Avoid nested panel blocks. This is the case for vertical_tabs.
      if (($element[$child]['#type'] ?? '') != 'details') {
        $element[$child]['#theme_wrappers'][] = 'container__panel_block';
      }
    }
    return $element;
  }

  /**
   * Correct theme wrappers set by preRenderDetails().
   *
   * The outer wrapper should always be the panel block container.
   * #pre_render callback.
   */
  public static function preRenderRadios($element) {
    $containerWrapperIndex = array_search('container__panel_block', $element['#theme_wrappers']);
    if ($containerWrapperIndex !== FALSE) {
      unset($element['#theme_wrappers'][$containerWrapperIndex]);
      $element['#theme_wrappers'][] = 'container__panel_block';
    }
    return $element;
  }

  /**
   * Add proper actions to sets of actions and its buttons.
   *
   * #pre_render callback.
   */
  public static function preRenderActions($element) {
    foreach (Element::getVisibleChildren($element) as $child) {
      $classes = $element[$child]['#attributes']['class'] ?? [];
      if (in_array('button--danger', $classes, TRUE)) {
        $element[$child]['#attributes']['class'][] = 'is-danger';
      }
      if (!empty($element[$child]['#button_type'])) {
        $element[$child]['#attributes']['class'][] = 'is-' . $element[$child]['#button_type'];
      }

    }
    $element['#attributes']['class'][] = 'control';
    $element['#attributes']['class'][] = 'buttons';
    return $element;
  }

}
